@@ -1,19 +1,29 @@
 !
-! Copyright (C) 2005-2006   Christopher C. Hulbert
+! Copyright (C) 2005-2016   Christopher C. Hulbert
 !
-!  This library is free software; you can redistribute it and/or
-!  modify it under the terms of the GNU Lesser General Public
-!  License as published by the Free Software Foundation; either
-!  version 2.1 of the License, or (at your option) any later version.
+! All rights reserved.
 !
-!  This library is distributed in the hope that it will be useful,
-!  but WITHOUT ANY WARRANTY; without even the implied warranty of
-!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-!  Lesser General Public License for more details.
+! Redistribution and use in source and binary forms, with or without
+! modification, are permitted provided that the following conditions are met:
 !
-!  You should have received a copy of the GNU Lesser General Public
-!  License along with this library; if not, write to the Free Software
-!  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+!    1. Redistributions of source code must retain the above copyright notice,
+!       this list of conditions and the following disclaimer.
+!
+!    2. Redistributions in binary form must reproduce the above copyright
+!       notice, this list of conditions and the following disclaimer in the
+!       documentation and/or other materials provided with the distribution.
+!
+! THIS SOFTWARE IS PROVIDED BY CHRISTOPHER C. HULBERT ``AS IS'' AND ANY
+! EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+! WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+! DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER C. HULBERT OR CONTRIBUTORS BE
+! LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+! CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+! SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+! INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+! CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+! ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+! POSSIBILITY OF SUCH DAMAGE.
 !
 PROGRAM test
 USE MATIO
@@ -105,10 +115,10 @@ CONTAINS
 
         outfilename = 'test_mat_copy.mat'
         WRITE (*,'(a)',ADVANCE='NO') 'Enter the input filename: '
-        READ (*,*) inputfile
+        READ (*,'(a128)') inputfile
 
         err = FMat_Open(inputfile,MAT_ACC_RDWR,mat)
-        err = FMat_Create(outfilename,mat2)
+        err = FMat_Create(outfilename,MAT_FT_MAT5,mat2)
         IF ( err .NE. 0 ) THEN
             WRITE (*,*) 'Error opening file ',inputfile
             RETURN
@@ -360,9 +370,9 @@ CONTAINS
         CHARACTER(1),DIMENSION(:),ALLOCATABLE :: char_data
 
         WRITE (*,'(a)',ADVANCE='NO') 'Enter the input filename: '
-        READ (*,*) inputfile
+        READ (*,'(a128)') inputfile
         WRITE (*,'(a)',ADVANCE='NO') 'Enter the variable name: '
-        READ (*,*) varname
+        READ (*,'(a32)') varname
 
         err = FMat_Open(inputfile,MAT_ACC_RDONLY,mat)
         IF ( err .NE. 0 ) THEN
@@ -520,9 +530,9 @@ CONTAINS
         INTEGER(1),DIMENSION(4)   :: int8_data
 
         WRITE (*,'(a)',ADVANCE='NO') 'Enter the input filename: '
-        READ (*,*) inputfile
+        READ (*,'(a128)') inputfile
         WRITE (*,'(a)',ADVANCE='NO') 'Enter the variable name: '
-        READ (*,*) varname
+        READ (*,'(a32)') varname
 
         err = FMat_Open(inputfile,MAT_ACC_RDONLY,mat)
         IF ( err .NE. 0 ) THEN
