@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2012   Christopher C. Hulbert
+ * Copyright (C) 2005-2016   Christopher C. Hulbert
  *
  * All rights reserved.
  *
@@ -31,7 +31,7 @@
 #include <string.h>
 #include <math.h>
 #include <getopt.h>
-#include "matio_private.h"
+#include "matio.h"
 #if !defined(HAVE_STRCASECMP)
 #   define strcasecmp(a,b) strcmp(a,b)
 #endif
@@ -68,7 +68,7 @@ static const char *helpstr[] = {
 "mat_file          name of the MAT file to dump",
 "var1 var2 ...     If specified, dumps only listed variables",
 "",
-"Report bugs to <cch@isl-inc.com>.",
+"Report bugs to <chulbe2lsu@sourceforge.net>.",
 NULL
 };
 
@@ -592,12 +592,12 @@ print_default_number(enum matio_types type, void *data)
         case MAT_T_SINGLE:
             printf("%g",*(float*)data);
             break;
-#ifdef HAVE_MAT_INT64_T
+#ifdef _mat_int64_t
         case MAT_T_INT64:
             printf("%lld",*(mat_int64_t*)data);
             break;
 #endif
-#ifdef HAVE_MAT_UINT64_T
+#ifdef _mat_uint64_t
         case MAT_T_UINT64:
             printf("%llu",*(mat_uint64_t*)data);
             break;
@@ -849,8 +849,8 @@ main (int argc, char *argv[])
                 exit(EXIT_SUCCESS);
             case 'V':
                 printf("%s %s\nWritten by Christopher Hulbert\n\n"
-                       "Copyright(C) 2006-2012 Christopher C. Hulbert\n",
-                       prog_name,PACKAGE_VERSION);
+                       "Copyright(C) 2006-2016 Christopher C. Hulbert\n",
+                       prog_name,MATIO_VERSION_STR);
                 exit(EXIT_SUCCESS);
             default:
                 printf("%c not a valid option\n", c);
@@ -893,7 +893,6 @@ main (int argc, char *argv[])
                 }
                 (*printfunc)(matvar);
                 Mat_VarFree(matvar);
-                matvar = NULL;
             } else {
                 Mat_Warning("Couldn't find variable %s in the MAT file",
                       argv[i]);
@@ -905,13 +904,11 @@ main (int argc, char *argv[])
             while ( (matvar = Mat_VarReadNext(mat)) != NULL ) {
                 (*printfunc)(matvar);
                 Mat_VarFree(matvar);
-                matvar = NULL;
             }
         } else {
             while ( (matvar = Mat_VarReadNextInfo(mat)) != NULL ) {
                 (*printfunc)(matvar);
                 Mat_VarFree(matvar);
-                matvar = NULL;
             }
         }
     }
