@@ -45,23 +45,27 @@
 /* Have MAT int8 */
 #define HAVE_MAT_INT8_T 1
 
-/* Have MAT int16 */
+/* Have MAT uint16 */
 #define HAVE_MAT_UINT16_T 1
 
-/* Have MAT int32 */
+/* Have MAT uint32 */
 #define HAVE_MAT_UINT32_T 1
 
-/* Have MAT int64 */
+/* Have MAT uint64 */
 #define HAVE_MAT_UINT64_T 1
 
-/* Have MAT int8 */
+/* Have MAT uint8 */
 #define HAVE_MAT_UINT8_T 1
 
 /* Define to 1 if you have the <memory.h> header file. */
 #define HAVE_MEMORY_H 1
 
 /* Have snprintf */
+#if defined(_MSC_VER) && _MSC_VER >= 1900
+#define HAVE_SNPRINTF 1
+#else
 #undef HAVE_SNPRINTF
+#endif
 
 /* Define to 1 if you have the <stdlib.h> header file. */
 #define HAVE_STDLIB_H 1
@@ -85,10 +89,21 @@
 #undef HAVE_VASPRINTF
 
 /* Have va_copy */
+#if defined(_MSC_VER) && _MSC_VER >= 1800
+#define HAVE_VA_COPY 1
+#else
 #undef HAVE_VA_COPY
+#endif
 
 /* Have vsnprintf */
+#if defined(_MSC_VER) && _MSC_VER >= 1900
+#define HAVE_VSNPRINTF 1
+#if !defined(HAVE_C99_VSNPRINTF)
+#define HAVE_C99_VSNPRINTF 1
+#endif
+#else
 #undef HAVE_VSNPRINTF
+#endif
 
 /* Have va_copy */
 #undef HAVE___VA_COPY
@@ -101,10 +116,10 @@
 #undef LT_OBJDIR
 
 /* Platform */
-#if defined(_WIN32)
-#   define MATIO_PLATFORM "i686-pc-windows"
-#elif defined(_WIN64)
+#if defined(_WIN64)
 #   define MATIO_PLATFORM "x86_64-pc-windows"
+#elif defined(_WIN32)
+#   define MATIO_PLATFORM "i686-pc-windows"
 #endif
 
 /* Debug disabled */
@@ -120,7 +135,7 @@
 #define PACKAGE_NAME "MATIO"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "MATIO 1.5.2"
+#define PACKAGE_STRING "MATIO 1.5.3"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "matio"
@@ -129,7 +144,7 @@
 #define PACKAGE_URL "http://sourceforge.net/projects/matio"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "1.5.2"
+#define PACKAGE_VERSION "1.5.3"
 
 /* The size of `char', as computed by sizeof. */
 #define SIZEOF_CHAR 1
@@ -152,16 +167,16 @@
 /* The size of `short', as computed by sizeof. */
 #define SIZEOF_SHORT 2
 
-#if defined(_WIN32)
-    /* The size of `void *', as computed by sizeof. */
-#   define SIZEOF_VOID_P 4
-    /* The size of `size_t', as computed by sizeof. */
-#    define SIZEOF_SIZE_T 4
-#elif defined(_WIN64)
+#if defined(_WIN64)
     /* The size of `void *', as computed by sizeof. */
 #   define SIZEOF_VOID_P 8
     /* The size of `size_t', as computed by sizeof. */
 #    define SIZEOF_SIZE_T 8
+#elif defined(_WIN32)
+    /* The size of `void *', as computed by sizeof. */
+#   define SIZEOF_VOID_P 4
+    /* The size of `size_t', as computed by sizeof. */
+#    define SIZEOF_SIZE_T 4
 #endif
 
 /* Define to 1 if you have the ANSI C header files. */
@@ -171,7 +186,7 @@
 #define HAVE_CTYPE_H 1
 
 /* Version number of package */
-#define VERSION "1.5.1"
+#define VERSION "1.5.3"
 
 /* Z prefix */
 #undef Z_PREFIX
