@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013   Christopher C. Hulbert
+ * Copyright (C) 2010-2016   Christopher C. Hulbert
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -25,19 +25,30 @@
 #define MATIO_MINOR_VERSION 5
 
 /* Matio release level number */
-#define MATIO_RELEASE_LEVEL 2
+#define MATIO_RELEASE_LEVEL 3
 
 /* Matio version number */
-#define MATIO_VERSION 152
+#define MATIO_VERSION 153
+
+/* Matio version string */
+#define MATIO_VERSION_STR "1.5.3"
 
 /* Default file format */
 #define MAT_FT_DEFAULT MAT_FT_MAT5
 
-/* Define to 1 if you have the <stdint.h> header file. */
+/* Have the <stdint.h> header file */
+#if defined(_MSC_VER) && _MSC_VER >= 1600
+#define MATIO_HAVE_STDINT_H 1
+#else
 #undef MATIO_HAVE_STDINT_H
+#endif
 
-/* Define to 1 if you have the <inttypes.h> header file. */
+/* Have the <inttypes.h> header file */
+#if defined(_MSC_VER) && _MSC_VER >= 1800
+#define MATIO_HAVE_INTTYPES_H 1
+#else
 #undef MATIO_HAVE_INTTYPES_H
+#endif
 
 /* int16 type */
 #define _mat_int16_t short
@@ -51,16 +62,16 @@
 /* int8 type */
 #define _mat_int8_t signed char
 
-/* int16 type */
+/* uint16 type */
 #define _mat_uint16_t unsigned short
 
-/* int32 type */
+/* uint32 type */
 #define _mat_uint32_t unsigned
 
-/* int64 type */
+/* uint64 type */
 #define _mat_uint64_t unsigned long long
 
-/* int8 type */
+/* uint8 type */
 #define _mat_uint8_t unsigned char
 
 #if MATIO_HAVE_INTTYPES_H
